<?php

namespace Drupal\delay_submit\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Configure delay submit settings.
 */
class DelaySubmitSettingsForm extends ConfigFormBase {

  /**
   * Module handler service.
   */
  protected $moduleHandler;

  /**
   * Entity type manager.
   */
  protected $entityTypeManager;

  /**
   * Constructs a new DelaySubmitSettingsForm.
   */
  public function __construct(ModuleHandlerInterface $module_handler, EntityTypeManagerInterface $entity_type_manager) {
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['delay_submit.settings'];
  }

/**
 * {@inheritdoc}
 */
public function getFormId() {
    return 'delay_submit_settings_form';
}


  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('delay_submit.settings');

    if ($form_state->has('enabled_forms')) {
        $enabled_forms = $form_state->get('enabled_forms');
    } else {
        $enabled_forms = $config->get('enabled_forms') ?? [];
        $form_state->set('enabled_forms', $enabled_forms);
    }

    if ($form_state->get('num_forms') === NULL || $form_state->get('num_forms') < count($enabled_forms)) {
        $form_state->set('num_forms', count($enabled_forms));
    }

    $form['enabled_forms'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Configure Delay for Forms'),
        '#description' => $this->t('Specify which forms should have a submit button delay. Each form can have a different delay time.'),
        '#prefix' => '<div id="enabled-forms-wrapper">',
        '#suffix' => '</div>',
        '#tree' => TRUE,
    ];

    foreach ($enabled_forms as $i => &$entry) {
        if (!isset($entry['unique_id']) || empty($entry['unique_id'])) {
            $entry['unique_id'] = uniqid('form_', TRUE);
        }

        $form['enabled_forms'][$i]['unique_id'] = [
            '#type' => 'hidden',
            '#value' => $entry['unique_id'],
        ];

        $form['enabled_forms'][$i]['form_id'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Form ID'),
            '#description' => $this->t('Enter the machine name of the form you want to apply the delay to. Example: <code>user_login_form</code> for the login page.'),
            '#default_value' => $entry['form_id'] ?? '',
            '#autocomplete_route_name' => 'delay_submit.form_autocomplete',
            '#size' => 30,
        ];

        $form['enabled_forms'][$i]['delay_time'] = [
            '#type' => 'number',
            '#title' => $this->t('Delay Time (ms)'),
            '#description' => $this->t('Set the delay in milliseconds before the submit button appears. Minimum: 500ms, Maximum: 10,000ms.'),
            '#default_value' => isset($entry['delay_time']) ? $entry['delay_time'] : 2000,
            '#min' => 500,
            '#max' => 10000,
            '#step' => 100,
        ];
    }

    // ✅ Add a single remove button to remove the last entry
    if (!empty($enabled_forms)) {
        $form['enabled_forms']['remove_last'] = [
            '#type' => 'submit',
            '#value' => $this->t('Remove Last Entry'),
            '#description' => $this->t('Removes the last added form entry from the list.'),
            '#submit' => ['::removeLastEntry'],
            '#ajax' => [
                'callback' => '::addMoreCallback',
                'wrapper' => 'enabled-forms-wrapper',
            ],
        ];
    }

    // ✅ Ensure `enabled_forms` persists across AJAX updates
    $form_state->set('enabled_forms', $enabled_forms);

    $form['enabled_forms']['add_more'] = [
        '#type' => 'submit',
        '#value' => $this->t('Add Another Form'),
        '#description' => $this->t('Click to add another form entry.'),
        '#submit' => ['::addMore'],
        '#ajax' => [
            'callback' => '::addMoreCallback',
            'wrapper' => 'enabled-forms-wrapper',
            'event' => 'click',
        ],
    ];

    $form['fade_in_duration'] = [
        '#type' => 'number',
        '#title' => $this->t('Fade-In Duration (ms)'),
        '#description' => $this->t('Set the duration of the fade-in effect for the submit button after the delay time is reached. Default: 1000ms (1 second).'),
        '#default_value' => $config->get('fade_in_duration') ?? 1000,
        '#min' => 100,
        '#max' => 5000,
        '#step' => 100,
    ];

    return parent::buildForm($form, $form_state);
}







   /**
   * Handles form submission.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $enabled_forms = [];
    $form_values = $form_state->getValue('enabled_forms');

    // 🚨 Log raw form state input
    \Drupal::logger('delay_submit')->notice('🟢 Raw Form Values from FormState: @values', ['@values' => print_r($form_values, TRUE)]);

    if (!empty($form_values) && is_array($form_values)) {
        foreach ($form_values as $key => $values) {
            // 🚨 Log each row's values
            \Drupal::logger('delay_submit')->notice('🟢 Processing Row @key: @values', ['@key' => $key, '@values' => print_r($values, TRUE)]);

            if (is_array($values) && isset($values['form_id'], $values['delay_time'])) {
                $clean_form_id = trim($values['form_id']);
                $clean_delay_time = (int) $values['delay_time'];

                if (!empty($clean_form_id) && $clean_form_id !== 'delay_submit_settings_form') {
                    $enabled_forms[] = [
                        'form_id' => $clean_form_id,
                        'delay_time' => $clean_delay_time,
                    ];
                }
            }
        }
    }

    // 🚨 Log processed forms before saving
    \Drupal::logger('delay_submit')->notice('🟢 Final Processed Enabled Forms: @forms', ['@forms' => print_r($enabled_forms, TRUE)]);

    // 🚨 Check for empty results
    if (empty($enabled_forms) && !empty($form_values)) {
        \Drupal::logger('delay_submit')->error('🔴 Received form values but did not process correctly! Debug: @values', ['@values' => print_r($form_values, TRUE)]);
    }

    // 🚨 Only save if values exist
    if (!empty($enabled_forms)) {
        \Drupal::logger('delay_submit')->notice('🟢 Saving updated config: @forms', ['@forms' => print_r($enabled_forms, TRUE)]);

        // Save the configuration
        $this->configFactory->getEditable('delay_submit.settings')
            ->set('fade_in_duration', $form_state->getValue('fade_in_duration') ?? 1000)
            ->set('enabled_forms', $enabled_forms)
            ->save();

        // 🚀 **Clear the cache so new settings apply instantly**
        \Drupal::service('config.factory')->reset('delay_submit.settings');
        \Drupal::service('cache_tags.invalidator')->invalidateTags(['config:delay_submit.settings']);

        // 🚨 Log cache reset
        \Drupal::logger('delay_submit')->notice('🟢 Cache cleared: delay_submit.settings');
    } else {
        \Drupal::logger('delay_submit')->warning('🔴 No valid enabled forms detected, config not overwritten.');
    }

    // Persist values in form state
    $form_state->set('enabled_forms', $enabled_forms);
    $form_state->set('num_forms', count($enabled_forms));
    $form_state->setRebuild(TRUE);

    parent::submitForm($form, $form_state);
}

  /**
   * Adds more form fields dynamically.
   */
  public function addMore(array &$form, FormStateInterface $form_state) {
    // 🚀 Ensure `num_forms` is updated properly
    $num_forms = $form_state->get('num_forms') ?? 1;
    $num_forms++;
    $form_state->set('num_forms', $num_forms);

    // 🚀 Preserve existing form values
    $enabled_forms = $form_state->get('enabled_forms') ?? [];
    $enabled_forms[] = ['form_id' => '', 'delay_time' => 2000]; // Add an empty form entry
    $form_state->set('enabled_forms', $enabled_forms);

    // 🚀 Force a form rebuild to reflect changes
    $form_state->setRebuild(TRUE);
  }


  /**
   * AJAX callback to update the form fields dynamically.
   */
  public function addMoreCallback(array &$form, FormStateInterface $form_state) {
    return $form['enabled_forms'];
}


public function removeLastEntry(array &$form, FormStateInterface $form_state) {
    $enabled_forms = $form_state->get('enabled_forms') ?? [];

    if (!empty($enabled_forms)) {
        \Drupal::logger('delay_submit')->notice('🟢 BEFORE REMOVAL: Enabled forms = @forms', ['@forms' => print_r($enabled_forms, TRUE)]);

        // 🚀 Remove the last entry
        array_pop($enabled_forms);

        // 🚀 Save updates to form state
        $form_state->set('enabled_forms', $enabled_forms);
        $form_state->set('num_forms', count($enabled_forms));

        \Drupal::logger('delay_submit')->notice('🟢 AFTER REMOVAL: Enabled forms = @forms', ['@forms' => print_r($enabled_forms, TRUE)]);

        // 🚀 Force form to rebuild
        $form_state->setRebuild(TRUE);
    }
}

























}
